@@ -1,10 +1,9 @@
 use std::cmp;
-use std::cmp::Ordering;
 use std::collections::HashMap;
 use std::io::Write;
 
-use gitql_ast::expression::Expression;
-use gitql_ast::expression::ExpressionKind;
+use gitql_ast::expression::Expr;
+use gitql_ast::expression::ExprKind;
 use gitql_ast::statement::AggregateValue;
 use gitql_ast::statement::AggregationsStatement;
 use gitql_ast::statement::DoStatement;
@@ -16,7 +15,6 @@ use gitql_ast::statement::LimitStatement;
 use gitql_ast::statement::OffsetStatement;
 use gitql_ast::statement::OrderByStatement;
 use gitql_ast::statement::SelectStatement;
-use gitql_ast::statement::SortingOrder;
 use gitql_ast::statement::Statement;
 use gitql_ast::statement::StatementKind::*;
 use gitql_ast::statement::WhereStatement;
@@ -24,14 +22,15 @@ use gitql_core::environment::Environment;
 use gitql_core::object::GitQLObject;
 use gitql_core::object::Group;
 use gitql_core::object::Row;
-use gitql_core::types::DataType;
-use gitql_core::value::Value;
+use gitql_core::values::base::Value;
+use gitql_core::values::null::NullValue;
 
 use crate::data_provider::DataProvider;
 use crate::engine_evaluator::evaluate_expression;
 use crate::engine_filter::apply_filter_operation;
 use crate::engine_group::execute_group_by_statement;
 use crate::engine_join::apply_join_operation;
+use crate::engine_ordering::execute_order_by_statement;
 
 #[allow(clippy::borrowed_box)]
 pub fn execute_statement(
@@ -225,6 +224,7 @@ fn execute_select_statement(
     };
 
     gitql_object.groups.push(main_group);
+
     Ok(())
 }
 
@@ -234,7 +234,7 @@ fn execute_expression_selection(
     selected_rows: &mut [Row],
     object_titles: &[String],
     selected_expr_titles: &[String],
-    selected_expr: &[Box<dyn Expression>],
+    selected_expr: &[Box<dyn Expr>],
 ) -> Result<(), String> {
     // Cache the index of each expression position to provide fast insertion
     let mut titles_index_map: HashMap<String, usize> = HashMap::new();
@@ -251,13 +251,13 @@ fn execute_expression_selection(
             let expr_title = &selected_expr_titles[index];
             let value_index = *titles_index_map.get(expr_title).unwrap();
 
-            if index < row.values.len() && row.values[value_index].data_type() != DataType::Null {
+            if index < row.values.len() && !row.values[value_index].is_null() {
                 continue;
             }
 
             // Ignore evaluating expression if it symbol, that mean it a reference to aggregated value or function
-            let value = if expr.kind() == ExpressionKind::Symbol {
-                Value::Null
+            let value = if expr.kind() == ExprKind::Symbol {
+                Box::new(NullValue)
             } else {
                 evaluate_expression(env, expr, object_titles, &row.values)?
             };
@@ -358,63 +358,6 @@ fn execute_offset_statement(
     Ok(())
 }
 
-fn execute_order_by_statement(
-    env: &mut Environment,
-    statement: &OrderByStatement,
-    gitql_object: &mut GitQLObject,
-) -> Result<(), String> {
-    if gitql_object.is_empty() {
-        return Ok(());
-    }
-
-    if gitql_object.len() > 1 {
-        gitql_object.flat();
-    }
-
-    let main_group: &mut Group = &mut gitql_object.groups[0];
-    if main_group.is_empty() {
-        return Ok(());
-    }
-
-    main_group.rows.sort_by(|a, b| {
-        // The default ordering
-        let mut ordering = Ordering::Equal;
-
-        for i in 0..statement.arguments.len() {
-            let argument = &statement.arguments[i];
-            // No need to compare if the ordering argument is constants
-            if argument.is_const() {
-                continue;
-            }
-
-            // Compare the two set of attributes using the current argument
-            let first = &evaluate_expression(env, argument, &gitql_object.titles, &a.values)
-                .unwrap_or(Value::Null);
-            let other = &evaluate_expression(env, argument, &gitql_object.titles, &b.values)
-                .unwrap_or(Value::Null);
-
-            let current_ordering = first.compare(other);
-
-            // If comparing result still equal, check the next argument
-            if current_ordering == Ordering::Equal {
-                continue;
-            }
-
-            // Reverse the order if its not ASC order
-            ordering = if statement.sorting_orders[i] == SortingOrder::Descending {
-                current_ordering
-            } else {
-                current_ordering.reverse()
-            };
-            break;
-        }
-
-        ordering
-    });
-
-    Ok(())
-}
-
 fn execute_aggregation_function_statement(
     env: &mut Environment,
     statement: &AggregationsStatement,
@@ -450,9 +393,11 @@ fn execute_aggregation_function_statement(
                     .unwrap();
 
                 // Evaluate the Arguments to Values
-                let mut group_values: Vec<Vec<Value>> = Vec::with_capacity(group.rows.len());
+                let mut group_values: Vec<Vec<Box<dyn Value>>> =
+                    Vec::with_capacity(group.rows.len());
                 for object in &mut group.rows {
-                    let mut row_values: Vec<Value> = Vec::with_capacity(object.values.len());
+                    let mut row_values: Vec<Box<dyn Value>> =
+                        Vec::with_capacity(object.values.len());
                     for argument in arguments {
                         let value = evaluate_expression(
                             env,
@@ -561,11 +506,12 @@ fn execute_into_statement(
 }
 
 #[inline(always)]
-fn value_to_string_with_optional_enclosing(value: &Value, enclosed: &String) -> String {
+#[allow(clippy::borrowed_box)]
+fn value_to_string_with_optional_enclosing(value: &Box<dyn Value>, enclosed: &String) -> String {
     if enclosed.is_empty() {
-        return value.to_string();
+        return value.literal();
     }
-    format!("{}{}{}", enclosed, value, enclosed)
+    format!("{}{}{}", enclosed, value.literal(), enclosed)
 }
 
 pub fn execute_global_variable_statement(
